import { ComponentType, ComponentOptions } from "./utils";
export declare function createMixin(mixinFn: (options: ComponentOptions) => ComponentOptions): (ComponentType: ComponentType<any>) => ComponentType<any>;
export declare function compose(...fns: ((C: ComponentType<any>) => ComponentType<any>)[]): (C: ComponentType<any>) => ComponentType<any>;
